#
#  partition.py
#  ENRAGE
#
try:
    import psyco
    psyco.full()
except:
    pass

import sys, string
from commoncode import getMergedRegions, writeLog

versionString = '%s: version 1.2' % sys.argv[0]
print versionString

minFeature = 25
if len(sys.argv) < 5:
    print "usage: python %s mergeID regionfile1 regionfile2 combpartitionfile [-minFeature bp] [-padregion bp] [-mergeregion bp] [-nomerge] [-log altlogfile] [-locid] [-ignorerandom] [-chromField fieldNum]\n"
    print "       where regionfile2 can be set to NONE to only partition regionfile1 and"
    print "       -minFeature (default: %d bp) controls the size of the smallest partition\n" % minFeature
    sys.exit(1)

mergeID = sys.argv[1]
regionfilename1 = sys.argv[2]
regionfilename2 = sys.argv[3]
outfilename = sys.argv[4]
logfilename = 'partition.log'
locID = False
ignoreRandom = False

if '-minFeature' in sys.argv:
    minFeature = int(sys.argv[sys.argv.index('-minFeature') + 1])

cField = 1
if '-chromField' in sys.argv:
        cField = int(sys.argv[sys.argv.index('-chromField') + 1])

padregion = 0
if '-padregion' in sys.argv:
    padField = sys.argv.index('-padregion') + 1
    padregion = int(sys.argv[padField])
    print "padding %d bp on each side of a region" % padregion

mergeregion = 0
if '-mergeregion' in sys.argv:
    mergeField = sys.argv.index('-mergeregion') + 1
    mergeregion = int(sys.argv[mergeField])
    print "merging regions closer than %d bp" % mergeregion

merging = True
if '-nomerge' in sys.argv:
    merging = False

if '-log' in sys.argv:
    logfilename = sys.argv[sys.argv.index('-log') + 1]

if '-locid' in sys.argv:
    locID = True
    print "using locations as region ID"

if '-norandom' in sys.argv:
    ignoreRandom = True
    print "ignoring 'random' chromosomes"

writeLog(logfilename, versionString, string.join(sys.argv[1:]))

region1Dict = getMergedRegions(regionfilename1, maxDist = mergeregion,  minHits=-1, fullChrom = True, verbose = True, chromField = cField, doMerge=merging, pad=padregion)
if regionfilename2 != 'NONE':
    region2Dict = getMergedRegions(regionfilename2, maxDist = mergeregion,  minHits=-1, fullChrom = True, verbose = True, chromField = cField, doMerge=merging, pad=padregion)
else:
    region2Dict = {}

outregionDict = {}

chromList1 = set(sorted(region1Dict))
chromList2 = set(sorted(region2Dict))

for chrom in (chromList1 - chromList2):
    if ignoreRandom and 'random' in chrom:
        continue
    outregionDict[chrom] = []
    start = 0
    for (rstart, rstop, rlength) in region1Dict[chrom]:
        if (rstart - start) > minFeature and rlength > minFeature:
            outregionDict[chrom].append((start, rstart - 1, rstart - 1 - start))
            outregionDict[chrom].append((rstart, rstop - 1, rlength - 1))
            start = rstop
        elif (rstart - rstop) > minFeature:
            outregionDict[chrom].append((start, rstop - 1, rstop - 1 - start))
            start = rstop

for chrom in (chromList2 - chromList1):
    if ignoreRandom and 'random' in chrom:
        continue
    outregionDict[chrom] = []
    start = 0
    for (rstart, rstop, rlength) in region2Dict[chrom]:
        if (rstart - start) > minFeature and rlength > minFeature:
            outregionDict[chrom].append((start, rstart - 1, rstart - 1 - start))
            outregionDict[chrom].append((rstart, rstop - 1, rlength - 1))
            start = rstop
        elif (rstart - rstop) > minFeature:
            outregionDict[chrom].append((start, rstop - 1, rstop - 1 - start))
            start = rstop

for chrom in (chromList1 & chromList2):
    if ignoreRandom and 'random' in chrom:
        continue
    outregionDict[chrom] = []
    pointList = []
    for (rstart, rstop, rlength) in region1Dict[chrom]:
        pointList.append(rstart)
        pointList.append(rstop)
    for (rstart, rstop, rlength) in region2Dict[chrom]:
        pointList.append(rstart)
        pointList.append(rstop)
    pointList.sort()
    start = 0
    for point in pointList:
        if (point - start) > minFeature:
            outregionDict[chrom].append((start, point - 1, point - 1 - start))
            start = point
    

outfile = open(outfilename, 'w')
if locID:
    outfile.write('#chrom:start-stop\tchrom\tstart\tstop\tlength_kb\n')
else:
    outfile.write('#labelID\tchrom\tstart\tstop\tlength_kb\n')
index = 0
for chrom in outregionDict:
    for (start, stop, length) in outregionDict[chrom]:
        index += 1
        if locID:
            outfile.write("%s:%d-%d\t%s\t%d\t%d\t%.3f\n" % (chrom, start, stop, chrom, start, stop, length/1000.))
        else:
            outfile.write("%s%d\t%s\t%d\t%d\t%.3f\n" % (mergeID, index, chrom, start, stop, length/1000.))

message = "%s was partitioned into %d regions" % (mergeID, index)
print message
writeLog(logfilename, versionString, message)
