# set ERANGEPATH to the absolute or relative path to ERANGE, if it's not in the environment

if [ -z "$ERANGEPATH" ]
then
    ERANGEPATH='../commoncode'
fi

echo 'runWormAnalysis.sh: version 3.0'

# count the unique reads falling on the gene models ; the nomatch files are 
# mappable reads that fell outside of the Cistematic gene models and not the 
# unmappable of Eland (i.e, the "NM" reads)
python2.5 $ERANGEPATH/geneMrnaCounts.py celegans $1.rds $1.uniqs.count -markNM

# calculate a first-pass RPKM to re-weigh the unique reads,
# using 'none' for the splice count
python2.5 $ERANGEPATH/normalizeExpandedExonic.py celegans $1.rds $1.uniqs.count none $1.firstpass.rpkm

# recount the unique reads with weights calculated during the first pass
python2.5 $ERANGEPATH/geneMrnaCountsWeighted.py celegans $1.rds $1.firstpass.rpkm $1.uniqs.recount -uniq

# count splice reads
python2.5 $ERANGEPATH/geneMrnaCounts.py celegans $1.rds $1.splices.count -splices -noUniqs

# calculate expanded exonic read density
python2.5 $ERANGEPATH/normalizeExpandedExonic.py celegans $1.rds $1.uniqs.recount $1.splices.count $1.expanded.rpkm -gidField 0 -maxLength 0.5

# weigh multi-reads
python2.5 $ERANGEPATH/geneMrnaCountsWeighted.py celegans $1.rds $1.expanded.rpkm $1.multi.count -multi

# calculate final exonic read density
python2.5 $ERANGEPATH/normalizeFinalExonic.py celegans $1.rds $1.expanded.rpkm $1.multi.count $1.final.rpkm -multifraction
